@@ -6,11 +6,13 @@ from datetime import datetime
 from getpass import getpass
 import logging
 import hashlib
+import socket
 import json
 import re
 import mimetypes
 import calendar
 from contextlib import closing  # for Python2.6 compatibility
+from gzip import GzipFile
 
 import yaml
 from beefish import decrypt, encrypt_file
@@ -22,8 +24,9 @@ from bakthat.backends import GlacierBackend, S3Backend, RotationConfig
 from bakthat.conf import config, DEFAULT_DESTINATION, DEFAULT_LOCATION, CONFIG_FILE
 from bakthat.utils import _interval_string_to_seconds
 from bakthat.models import Backups, Inventory
+from bakthat.sync import BakSyncer
 
-__version__ = "0.4.4"
+__version__ = "0.4.5"
 
 app = aaargh.App(description="Compress, encrypt and upload files directly to Amazon S3/Glacier.")
 
@@ -116,6 +119,8 @@ def delete_older_than(filename, interval, destination=DEFAULT_DESTINATION, profi
         backup.set_deleted()
         deleted.append(real_key)
 
+    BakSyncer(conf).sync_auto()
+
     return deleted
 
 
@@ -180,6 +185,8 @@ def rotate_backups(filename, destination=DEFAULT_DESTINATION, profile="default",
             backup.set_deleted()
             deleted.append(real_key)
 
+    BakSyncer(conf).sync_auto()
+
     return deleted
 
 
@@ -212,6 +219,9 @@ def backup(filename=os.getcwd(), destination=None, prompt="yes", tags=[], profil
     :type conf: dict
     :keyword conf: Override/set AWS configuration.
 
+    :type custom_filename: str
+    :keyword custom_filename: Override the original filename (only in metadata)
+
     :rtype: dict
     :return: A dict containing the following keys: stored_filename, size, metadata, backend and filename.
 
@@ -227,7 +237,7 @@ def backup(filename=os.getcwd(), destination=None, prompt="yes", tags=[], profil
     stored_filename = backup_file_fmt.format(arcname, date_component)
 
     backup_date = int(now.strftime("%s"))
-    backup_data = dict(filename=arcname,
+    backup_data = dict(filename=kwargs.get("custom_filename", arcname),
                        backup_date=backup_date,
                        last_updated=backup_date,
                        backend=destination,
@@ -289,7 +299,8 @@ def backup(filename=os.getcwd(), destination=None, prompt="yes", tags=[], profil
 
     backup_data["tags"] = tags
 
-    backup_data["metadata"] = dict(is_enc=bakthat_encryption)
+    backup_data["metadata"] = dict(is_enc=bakthat_encryption,
+                                   client=socket.gethostname())
     backup_data["stored_filename"] = stored_filename
 
     access_key = storage_backend.conf.get("access_key")
@@ -308,6 +319,8 @@ def backup(filename=os.getcwd(), destination=None, prompt="yes", tags=[], profil
     # Insert backup metadata in SQLite
     Backups.create(**backup_data)
 
+    BakSyncer(conf).sync_auto()
+
     return backup_data
 
 
@@ -354,38 +367,39 @@ def _display_backups(backups):
 @app.cmd(help="Set AWS S3/Glacier credentials.")
 @app.cmd_arg('-p', '--profile', type=str, default="default", help="profile name (default by default)")
 def configure(profile="default"):
-    new_conf = config.copy()
-    new_conf[profile] = config.get(profile, {})
+    try:
+        new_conf = config.copy()
+        new_conf[profile] = config.get(profile, {})
 
-    new_conf_file = open(CONFIG_FILE, "w")
+        new_conf[profile]["access_key"] = raw_input("AWS Access Key: ")
+        new_conf[profile]["secret_key"] = raw_input("AWS Secret Key: ")
+        new_conf[profile]["s3_bucket"] = raw_input("S3 Bucket Name: ")
+        new_conf[profile]["glacier_vault"] = raw_input("Glacier Vault Name: ")
 
-    new_conf[profile]["access_key"] = raw_input("AWS Access Key: ")
-    new_conf[profile]["secret_key"] = raw_input("AWS Secret Key: ")
-    new_conf[profile]["s3_bucket"] = raw_input("S3 Bucket Name: ")
-    new_conf[profile]["glacier_vault"] = raw_input("Glacier Vault Name: ")
-
-    while 1:
-        default_destination = raw_input("Default destination ({0}): ".format(DEFAULT_DESTINATION))
-        if default_destination:
-            default_destination = default_destination.lower()
-            if default_destination in ("s3", "glacier"):
-                break
+        while 1:
+            default_destination = raw_input("Default destination ({0}): ".format(DEFAULT_DESTINATION))
+            if default_destination:
+                default_destination = default_destination.lower()
+                if default_destination in ("s3", "glacier"):
+                    break
+                else:
+                    log.error("Invalid default_destination, should be s3 or glacier, try again.")
             else:
-                log.error("Invalid default_destination, should be s3 or glacier, try again.")
-        else:
-            default_destination = DEFAULT_DESTINATION
-            break
+                default_destination = DEFAULT_DESTINATION
+                break
 
-    new_conf[profile]["default_destination"] = default_destination
-    region_name = raw_input("Region Name ({0}): ".format(DEFAULT_LOCATION))
-    if not region_name:
-        region_name = DEFAULT_LOCATION
-    new_conf[profile]["region_name"] = region_name
+        new_conf[profile]["default_destination"] = default_destination
+        region_name = raw_input("Region Name ({0}): ".format(DEFAULT_LOCATION))
+        if not region_name:
+            region_name = DEFAULT_LOCATION
+        new_conf[profile]["region_name"] = region_name
 
-    yaml.dump(new_conf, new_conf_file, default_flow_style=False)
+        yaml.dump(new_conf, open(CONFIG_FILE, "w"), default_flow_style=False)
 
-    log.info("Config written in %s" % CONFIG_FILE)
-    log.info("Run bakthat configure_backups_rotation if needed.")
+        log.info("Config written in %s" % CONFIG_FILE)
+        log.info("Run bakthat configure_backups_rotation if needed.")
+    except KeyboardInterrupt:
+        log.error("Cancelled by user")
 
 
 @app.cmd(help="Configure backups rotation")
@@ -453,7 +467,7 @@ def restore(filename, destination=DEFAULT_DESTINATION, profile="default", **kwar
     log.info("Restoring " + key_name)
 
     # Asking password before actually download to avoid waiting
-    if key_name and key_name.endswith(".enc"):
+    if key_name and backup.is_encrypted():
         password = kwargs.get("password")
         if not password:
             password = getpass()
@@ -471,7 +485,7 @@ def restore(filename, destination=DEFAULT_DESTINATION, profile="default", **kwar
         # If it's a job_check call, we return Glacier job data
         return out
 
-    if out and key_name.endswith(".enc"):
+    if out and backup.is_encrypted():
         log.info("Decrypting...")
         decrypted_out = tempfile.TemporaryFile()
         decrypt(out, decrypted_out, password)
@@ -480,9 +494,14 @@ def restore(filename, destination=DEFAULT_DESTINATION, profile="default", **kwar
     if out:
         log.info("Uncompressing...")
         out.seek(0)
-        tar = tarfile.open(fileobj=out)
-        tar.extractall()
-        tar.close()
+        if not backup.metadata.get("KeyValue"):
+            tar = tarfile.open(fileobj=out)
+            tar.extractall()
+            tar.close()
+        else:
+            with closing(GzipFile(fileobj=out, mode="r")) as f:
+                with open(backup.stored_filename, "w") as out:
+                    out.write(f.read())
 
         return True
 
@@ -534,9 +553,25 @@ def delete(filename, destination=DEFAULT_DESTINATION, profile="default", **kwarg
     storage_backend.delete(key_name)
     backup.set_deleted()
 
+    BakSyncer(conf).sync_auto()
+
     return True
 
 
+@app.cmd(help="Trigger synchronization")
+def sync(**kwargs):
+    """Trigger synchronization."""
+    conf = kwargs.get("conf")
+    BakSyncer(conf).sync()
+
+
+@app.cmd(help="Reset synchronization")
+def reset_sync(**kwargs):
+    """Reset synchronization."""
+    conf = kwargs.get("conf")
+    BakSyncer(conf).reset_sync()
+
+
 @app.cmd(help="List stored backups.")
 @app.cmd_arg('-d', '--destination', type=str, help="s3|glacier")
 @app.cmd_arg('-p', '--profile', type=str, default="default", help="profile name (default by default)")
